/*
 * module2.c
 * Basic demo program for second DSP in 2 DSP HERON-API example.
 * Before running this program, take a look at the 'network' file, 
 * it contains information describing the actual hardware configuration
 * and should be updated before running the server.
 *
 *     PW    ver 2.0    30-3-00   changed for heron_api v2.3
 *     JT    ver 2.1  19-7-00    changed for server/loader 3.2
 */


#include "heron1_pc8.h"
#include <std.h>
#include <swi.h>

/* YOU need to set this value to reflect how your module is connected
to the first module. */
#define FIFONO	0	/* FIFO through which module communicates with the first module */

/* DSP/BIOS treats main() as an initialisation routine. While main() */
/* is being executed, DSP/BIOS is initialised but not running: all   */
/* hardware and software interrupts are disabled. This is a good     */
/* place to run bootloader(), since no other threads/tasks will be   */
/* running; tasks or threads that may concurrently try to communica- */
/* with processors that have not yet been booted.                    */

main()
{
	/* 
	 * The next line insures that the program is properly loaded onto
	 * the root DSP. The bootloader() function will call an external
	 * function "init_io_functions". This function is implemented in 
	 * a file "stub.c" in this directory. Please include this file
	 * into your project, and make sure that the correct heron.h
	 * file is included in this "stub.c" file .
	 */
	bootloader();
}

void maintask(void)
{

	HeronFIFO *rfifo;
	HeronFIFO *wfifo;
	int status;
	int data;
	int pattern[32];

	/*
	 * All processors in the system are required to clear the config line
	 * to an in-active state. One all processors in a system performed a
	 * HeronConfigOff(), the config line goes in-active, and GDIO modules
	 * are only then able to send data onto C6xxx modules. 
	 * If you only use processor modules but no GDIO modules, then this
	 * doesn't matter to you and you can leave the config line in any
	 * state (active -> don't do a HeronConfigOff() or in-active -> do a
	 * HeronConfigOff() 
	 */
	 
    HeronConfigOff();
	/*initialise digital I/Os*/
	HeronDigioOut(0x0);

	/* open the FIFO for reading*/
    rfifo=HeronOpenFifo(FIFONO,"r");
    if (rfifo==NULL)
     {
     HeronDigioOut(0x1); /* assert digital I/O bit 0 to show the error*/
     exit(-1); 	       
     }
/* open the FIFO for writing*/
    wfifo=HeronOpenFifo(FIFONO,"w");
    if (wfifo==NULL)
     {
     HeronDigioOut(0x2); /* assert digital I/O bit 1 to show the error*/
     exit(-1); 	
     } 

	
/* send a word to the first DSP */
	data = 0x2;
  	status = HeronWrite(wfifo,&data,1);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(wfifo);
	}
        if (status != HERON_OK) 
	{
	HeronDigioOut(0x8); /* assert digital I/O bit 3 to show the error*/
	exit(-1);
	}

/* read a pattern from the first DSP */
  	status = HeronRead(rfifo,pattern,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(rfifo);
	}
        if (status != HERON_OK) 
	{
	HeronDigioOut(0x4); /* assert digital I/O bit 2 to show the error*/
	exit(-1);
	}
	
/* send the receievd pattern back to the first DSP */

  	status = HeronWrite(wfifo,pattern,32);
	if (status == HERON_IO_IN_PROGRESS)
	{
	status = HeronWaitIo(wfifo);
	}
        if (status != HERON_OK) 
	{
	HeronDigioOut(0x8); /* assert digital I/O bit 3 to show the error*/
	exit(-1);
	}

/* close the fifos*/
	status = HeronClose(rfifo);
	if (status != HERON_OK)
	{
	HeronDigioOut(0x9); /* assert digital I/O to show the error*/
	exit(-1);
	}
	status = HeronClose(wfifo);
	if (status != HERON_OK)
	{
	HeronDigioOut(0x9); /* assert digital I/O bit 3 to show the error*/
	exit(-1);
	}




	while (1) {}            /* No real purpose, just for tracking */
}
/*** End Of File ***/
