
#if (WIN32 && CCSTUDIO)
#include <afxdisp.h>
#endif

#ifdef _LINUX
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#else
#ifdef _MSDOS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#else
#ifdef _RTOS32
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <rttarget.h>
#include <rttbios.h>
#include <rtk32.h>
#include <rtkeybrd.h>
extern "C" { char getch(void); }
#else
//#include <stdafx.h>
#endif
#endif
#endif

#include "heapi.h"

int				bnumber;				// board number
char			*bname = NULL;			// board name (API compliant)
int				comport = 0;			// device (hep8a: only 0)
HE_DWORD		status;
HE_HANDLE		hDevice;
HE_IOSTATUS		ReadIoStatus;
HE_IOSTATUS		WriteIoStatus;
HE_MEMHANDLE	memHandle = NULL;
HE_PDWORDBUFFER test;

void api_read(HE_INT32 Count, unsigned char *Buffer)
{
	HE_DWORD	status;
    
    status = HeRead(hDevice, (void *)Buffer, Count, ReadIoStatus);
	if (status == HE_IoInProgress)
	{
        status = HeWaitForIo(hDevice, ReadIoStatus);
    }

	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Error reading board \"%s\" (%s).", bname, mess);
		exit(0);
    }
}

int api_write(HE_INT32 Count, unsigned char *Buffer)
{
	HE_DWORD status;

    status = HeWrite(hDevice, (void *)Buffer, Count, WriteIoStatus);
    if (status == HE_IoInProgress)
	{
        status = HeWaitForIo(hDevice, WriteIoStatus);
    }

	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Error writing board \"%s\" (%s).", bname, mess);
		exit(0);
    }

	return 0;
}

void DoApi(char *brdname, int bno, int device)
{
	hDevice = NULL;
	bname	= brdname;
	bnumber	= bno;
	comport = device;
	status  = HeOpen(bname, bnumber, device, &hDevice);
	if (status != HE_OK)
	{	
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot open \"%s\", device %d (%s).", bname, device, mess);
		exit(0);
	}

    ReadIoStatus = NULL;
    status = HeInitIoStatus(hDevice, &ReadIoStatus);
	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot init read structure on \"%s\" (%s).", bname, mess);
		exit(0);
    }

	// Initialise write status structure
    WriteIoStatus = NULL;
    status = HeInitIoStatus(hDevice, &WriteIoStatus);
	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot init write structure on \"%s\" (%s).", bname, mess);
		exit(0);
    }

	status = HeAlloc(&memHandle, 1024);			// allocate memory
	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot allocate memhandle for \"%s\" (%s).", bname, mess);
		exit(0);
    }

	status = HeLock(memHandle, &test);				// get the address
	if (status != HE_OK)
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot lock memhandle for \"%s\" (%s).", bname, mess);
		exit(0);
    }

	printf("Write word. This will make the config light flash.\n");
	test[0] = 0x11223344;
	api_write(4, (unsigned char *) test);

	printf("Read back the message word.\n");

	api_read(4, (unsigned char *) test);
	printf("message received was %x\n", test[0]);

	status = HeClose(&hDevice);
	if (status != HE_OK )
	{
		char mess[256];
		HeErr2Text(status, mess);
		printf("Cannot close board \"%s\" (%s).", bname, mess);
	}
}

void main(int argc, char *argv[])
{
	int bdno, fifo;

#ifdef _RTOS32
	// Initialize the kernel and keyboard driver
	RTKernelInit(5);
	//   CLKSetTimerIntVal(10*1000); // 10 millisecond timer tick
	KBInit();
#endif

	if (argc<4)
	{
		printf("\n");
		printf("Syntax:   mysl <board type> <board number> <fifo>\n\n");
		printf("Example:  mysl     hep9a          0           0\n\n");
		return;
	}

	bdno = atoi(argv[2]);
	fifo = atoi(argv[3]);
	DoApi(argv[1], bdno /*board number*/, fifo /* device: hep8a has only 0*/);

	printf("End.\n");
}
